<script type="text/javascript">
     window.NSOT_VERSION = '{{NSOT_VERSION}}';
</script>

<script src="/static/build/vendor/angular/angular.min.js"></script>
<script src="/static/build/vendor/Chart.js/Chart.js"></script>
<script src="/static/build/vendor/angular-route/angular-route.min.js"></script>
<script src="/static/build/vendor/angular-resource/angular-resource.min.js"></script>
<script src="/static/build/vendor/ng-tags-input/ng-tags-input.min.js"></script>
<script src="/static/build/vendor/lodash/lodash.min.js"></script>
<script src="/static/build/vendor/jquery/dist/jquery.min.js"></script>
<script src="/static/build/vendor/bootstrap/dist/js/bootstrap.min.js"></script>
<script src="/static/build/vendor/moment/min/moment.min.js"></script>
<script src="/static/build/vendor/angular-chart.js/dist/angular-chart.js"></script>

<script src="/static/build/js/app.min.js"></script>
